package animation;

import java.util.Iterator;
import java.util.Stack;

import animation.StackFrame.FrameType;

public class Memory {

    private StackFrame global;
    private Stack< StackFrame > stack;
    
    public Memory()
    {
        stack = new Stack<StackFrame>();
        global = new StackFrame( FrameType.FUNCTION );
    }
    
    public void addFrame( StackFrame frame )
    {
        stack.push( frame );
    }
    
    public StackFrame removeFrame()
    {
        return stack.pop();
    }
    
    public <T> void declare( String name, T value, boolean global )
    {
        if( global )
            this.global.declare( name, value );
        else
            stack.peek().declare( name, value );
    }
    
    public <T> void write( String name, T value, boolean global )
    {
        if( global )
            this.global.set( name, value );
        else
        {
        	Iterator<StackFrame> iterator = stack.iterator();
        	while (iterator.hasNext()) {
        	   StackFrame stackF = iterator.next();
        	   if( stackF.isDefined( name ) )
        	   {
        		   stackF.set( name, value );
        		   return;
        	   }
        	   if( stackF.getType() == FrameType.FUNCTION )
        		   break;
        	}
        }
    }
    
    public <T> T read( String name, boolean global )
    {
        if( global )
            return this.global.get( name );
        else
        {
        	Iterator<StackFrame> iterator = stack.iterator();
        	while (iterator.hasNext()) {
        	   StackFrame stackF = iterator.next();
        	   if( stackF.isDefined( name ) )
        		   stackF.get( name );
        	   if( stackF.getType() == FrameType.FUNCTION )
        		   break;
        	}
            return null;
        }
    }
    
    public boolean isSomewhereDefined( String name, boolean global )
    {
    	if( global )
    		return this.global.isDefined( name );
    	Iterator<StackFrame> iterator = stack.iterator();
    	while (iterator.hasNext()) {
    	   StackFrame stackF = iterator.next();
    	   if( stackF.isDefined( name ) )
    		   return true;
    	   if( stackF.getType() == FrameType.FUNCTION )
    		   break;
    	}
    	return false;
    }
    
    public boolean isDefined( String name, boolean global )
    {
    	if( global )
    		return this.global.isDefined( name );
    	return stack.peek().get( name );
    }
    
    public void undeclare( String name, boolean global )
    {
        if( global )
            this.global.undeclare( name );
        else
            stack.peek().undeclare( name );
    }
}
