package bk;

import javax.swing.JFrame;
import javax.swing.JTree;

import animation.AnimatedAlgorithm;
import animation.AnimationController;
import animation.CodeLine;
import animation.ControlFlow;
import animation.Memory;
import animation.PseudoCodeNode;
import graph.LayeredGraphNode;
import lib.TextLayoutHelper;

/**
 * The main stage of the BK node placement algorithm.
 * @author kolja
 *
 */
public class BKNodePlacement extends AnimatedAlgorithm {

    /*
     * Private data structures to store the process of the algorithm
     */

    public enum State
    {
        CONFLICTS,
        LAYOUT1,
        LAYOUT2,
        LAYOUT3,
        LAYOUT4,
        COMBINE
    }

    private ConflictDetection conftion;
    private State state;
    private ExtremalLayoutCalc layouts[];
    private Combine combine;

    public BKNodePlacement(AnimationController controller, LayeredGraphNode graph, JFrame view) {
        super(controller, graph, view);
        state = State.CONFLICTS;
        conftion = new ConflictDetection( graph, this );
        layouts = new ExtremalLayoutCalc[ 4 ];
        layouts[ 0 ] = new ExtremalLayoutCalc( graph, this );
        layouts[ 1 ] = new ExtremalLayoutCalc( graph, this );
        layouts[ 2 ] = new ExtremalLayoutCalc( graph, this );
        layouts[ 3 ] = new ExtremalLayoutCalc( graph, this );
        combine = new Combine( graph, this );
    }
    
    public State getAlgorithmState()
    {
        return state;
    }

    @Override
    public PseudoCodeNode createPseudocodeTree( JTree tree )
    {
    	root = new PseudoCodeNode( TextLayoutHelper.setupPseudoCodeStage("BK Node Placement Algorithm" ), tree, new CodeLine() {

			@Override
			public ControlFlow runForward(Memory m) {
				return new ControlFlow( ControlFlow.STEP_INTO );
			}
        	
        }, this );
        root.setSelected( true );
    	String[] vars = { "graph" };
    	PseudoCodeNode mainFunction = new PseudoCodeNode( TextLayoutHelper.setupPseudoCode("function bkNodePlacement( graph )", vars ), tree, new CodeLine() {

			@Override
			public ControlFlow runForward(Memory m) {
				return new ControlFlow( ControlFlow.STEP_INTO );
			}
        	
        }, this );
    	
        PseudoCodeNode conflictDetectionFunction = conftion.createPseudocodeTree( tree );
        
        PseudoCodeNode node1 = new PseudoCodeNode( TextLayoutHelper.setupPseudoCode( "call detectConflicts( graph )", vars ), tree, new CodeLine() {

			@Override
			public ControlFlow runForward(Memory m) {
				if( !m.isDefined( "line_" + lineId + "_call", false ) )
				{
					m.declare( "line_" + lineId + "_call", true, false );
					return new ControlFlow( conflictDetectionFunction );
				}
				else
				{
					m.undeclare( "line_" + lineId + "_call", false );
					return new ControlFlow( ControlFlow.STEP_OVER );
				}
			}
        	
        }, this );
        root.add( mainFunction );
        mainFunction.add( node1 );
        root.add( conflictDetectionFunction );
        return root;
    }

    @Override
    public String getDebugString()
    {
        switch( state )
        {
        case CONFLICTS:
            return conftion.getDebugString();
        case LAYOUT1:
            return layouts[ 0 ].getDebugString();
        case LAYOUT2:
            return layouts[ 1 ].getDebugString();
        case LAYOUT3:
            return layouts[ 2 ].getDebugString();
        case LAYOUT4:
            return layouts[ 3 ].getDebugString();
        case COMBINE:
            return combine.getDebugString();
        }
        return "";
    }
}
