package bk;

import java.util.ArrayList;
import java.util.Stack;

import javax.swing.JTree;

import animation.AlgorithmStage;
import animation.AnimatedAlgorithm;
import animation.CodeLine;
import animation.ControlFlow;
import animation.Memory;
import animation.PseudoCodeNode;
import animation.PseudoCodeNode.CodeAction;
import graph.LayeredGraphEdge;
import graph.LayeredGraphNode;
import lib.TextLayoutHelper;

/**
 * the preprocessing stage of the bk node placement algorithm
 * 
 * @author kolja and eren
 *
 */
public class ConflictDetection implements AlgorithmStage {

    private LayeredGraphNode graph;
    private AnimatedAlgorithm alg;

    public ConflictDetection(LayeredGraphNode graph, AnimatedAlgorithm alg) {
        this.graph = graph;
        this.alg = alg;
    }

    @Override
    public PseudoCodeNode createPseudocodeTree(JTree tree) {
        String vars[] = { "i", "L", "k0", "l", "l1", "k1", "k", "v", "graph" };
        PseudoCodeNode root = new PseudoCodeNode(TextLayoutHelper.setupPseudoCode("mark_conflicts( graph )", vars), tree, new CodeLine() {

			@Override
			public ControlFlow runForward(Memory m) {
				if( !m.isDefined( "param1", true ) )
					return new ControlFlow( ControlFlow.STEP_OVER );
				//m.addFrame();
				LayeredGraphNode param = m.<LayeredGraphNode>read( "param1", true );
				m.undeclare( "param1", true );
				m.declare( "graph", param, false );
				m.declare( "Layers", param.getContainedLayers(), false );
				return new ControlFlow( ControlFlow.STEP_INTO );
			}
        }, alg );
        PseudoCodeNode[] lines = new PseudoCodeNode[16];
        lines[ 0 ] = new PseudoCodeNode( TextLayoutHelper.setupPseudoCode( "for i=1 to |L|-2 do", vars ), tree, new CodeLine() {

			@Override
			public ControlFlow runForward(Memory m) {
				// TODO Auto-generated method stub
				return null;
			}
        	
        }, alg );
        root.add( lines[ 0 ] );
        lines[ 1 ] = new PseudoCodeNode( TextLayoutHelper.setupPseudoCode( "k0 = 0; l = 0;", vars ), tree, new CodeLine() {

			@Override
			public ControlFlow runForward(Memory m) {
				// TODO Auto-generated method stub
				return null;
			}
        	
        }, alg);
        lines[ 0 ].add( lines[ 1 ] );
        /*
        lines[ 2 ] = new PseudoCodeNode( TextLayoutHelper.setupPseudoCode( "for l1=0 to |L[i+1]|-1 do", vars ), tree );
        lines[ 0 ].add( lines[ 2 ] );
        lines[ 3 ] = new PseudoCodeNode( TextLayoutHelper.setupPseudoCode( "if l1==|L[i+1]|-1 or L[i+1][l1] incident to inner segment between L[i+1] and L[i] then", vars ), tree );
        lines[ 2 ].add( lines[ 3 ] );
        lines[ 4 ] = new PseudoCodeNode( TextLayoutHelper.setupPseudoCode( "k1 = |L[i]|-1;", vars ), tree );
        lines[ 3 ].add( lines[ 4 ] );
        lines[ 5 ] = new PseudoCodeNode( TextLayoutHelper.setupPseudoCode( "if L[i+1][l1] incident to inner segment between L[i+1] and L[i] then", vars ), tree );
        lines[ 3 ].add( lines[ 5 ] );
        lines[ 6 ] = new PseudoCodeNode( TextLayoutHelper.setupPseudoCode( "k1 = pos(pred(L[i+1][l1])[0]);", vars ), tree );
        lines[ 5 ].add( lines[ 6 ] );
        lines[ 8 ] = new PseudoCodeNode( TextLayoutHelper.setupPseudoCode( "while l <= l1 do", vars ), tree );
        lines[ 3 ].add( lines[ 8 ] );
        lines[ 9 ] = new PseudoCodeNode( TextLayoutHelper.setupPseudoCode( "foreach v in pred(L[i+1][l]) do", vars ), tree );
        lines[ 8 ].add( lines[ 9 ] );
        lines[ 10 ] = new PseudoCodeNode( TextLayoutHelper.setupPseudoCode( "if k < k0 or k > k1 then mark segment (v,L[i+1][l]);", vars ), tree );
        lines[ 9 ].add( lines[ 10 ] );
        lines[ 12 ] = new PseudoCodeNode( TextLayoutHelper.setupPseudoCode( "l = l+1;", vars ), tree );
        lines[ 8 ].add( lines[ 12 ] );
        lines[ 14 ] = new PseudoCodeNode( TextLayoutHelper.setupPseudoCode( "k0 = k1;", vars ), tree );
        lines[ 3 ].add( lines[ 14 ] );
        lines[ 15 ] = new PseudoCodeNode( TextLayoutHelper.setupPseudoCodeStage( "mark conflicts in subgraphs" ), tree );
        root.add( lines[ 15 ] );*/
        return root;
    }

    @Override
    public String getDebugString() {/*
        String info = "| i  | l  | l1 | k0 | k1 | k  |\n";
        info +=       "|----|----|----|----|----|----|\n";
        info += "|" + TextLayoutHelper.strToLen( "" + i, 4 ) + 
                "|" + TextLayoutHelper.strToLen( "" + l, 4 ) + 
                "|" + TextLayoutHelper.strToLen( "" + l1, 4 ) + 
                "|" + TextLayoutHelper.strToLen( "" + k0, 4 ) + 
                "|" + TextLayoutHelper.strToLen( "" + k1, 4 ) + 
                "|" + TextLayoutHelper.strToLen( "" + k, 4 ) + "|\n";
        if( insideSubgraph && vIndex < graph.getContainedNodes().size() )
        {
            info += "Subgraph of " + graph.getContainedNodes().get( vIndex ).getName() + ":\n";
            String tmp = subgraphAlgs.get( vIndex ).getDebugString();
            info += tmp;
            return info;
        }
        return info;*/
    	return "";
    }

}
